package slimeknights.tconstruct.world.block;

import net.minecraft.block.Block;
import net.minecraft.block.BlockSapling;
import net.minecraft.block.properties.PropertyEnum;
import net.minecraft.block.state.BlockState;
import net.minecraft.block.state.IBlockState;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.util.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.common.EnumPlantType;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import java.util.List;
import java.util.Random;

import slimeknights.tconstruct.library.TinkerRegistry;
import slimeknights.tconstruct.world.TinkerWorld;
import slimeknights.tconstruct.world.block.BlockSlimeGrass.FoliageType;
import slimeknights.tconstruct.world.worldgen.SlimeTreeGenerator;

public class BlockSlimeSapling extends BlockSapling {
  public static PropertyEnum<FoliageType> FOLIAGE = BlockSlimeGrass.FOLIAGE;

  public BlockSlimeSapling() {
    func_149647_a(TinkerRegistry.tabWorld);
    func_180632_j(this.field_176227_L.func_177621_b());
    this.func_149672_a(field_149779_h);
  }

  @Override
  public void func_149666_a(Item itemIn, CreativeTabs tab, List<ItemStack> list) {
    for(FoliageType type : FoliageType.values()) {
      list.add(new ItemStack(this, 1, func_176201_c(func_176223_P().func_177226_a(FOLIAGE, type))));
    }
  }


  @Override
  protected BlockState func_180661_e() {
    // TYPE has to be included because of the BlockSapling constructor.. but it's never used.
    return new BlockState(this, FOLIAGE, field_176479_b, field_176480_a);
  }

  /**
   * Convert the given metadata into a BlockState for this Block
   */
  public IBlockState func_176203_a(int meta)
  {
    if(meta < 0 || meta >= BlockSlimeGrass.FoliageType.values().length) {
      meta = 0;
    }
    BlockSlimeGrass.FoliageType grass = BlockSlimeGrass.FoliageType.values()[meta];
    return this.func_176223_P().func_177226_a(BlockSlimeGrass.FOLIAGE, grass);
  }

  /**
   * Convert the BlockState into the correct metadata value
   */
  public int func_176201_c(IBlockState state)
  {
    return ((BlockSlimeGrass.FoliageType)state.func_177229_b(BlockSlimeGrass.FOLIAGE)).ordinal();
  }

  @Override
  public int func_180651_a(IBlockState state) {
    return func_176201_c(state);
  }

  @Override
  @SideOnly(Side.CLIENT)
  public int func_149635_D ()
  {
    return 0xffffff;
  }

  // Used for the item
  @SideOnly(Side.CLIENT)
  @Override
  public int func_180644_h(IBlockState state) {
    return 0xffffff;
  }

  // Used for the block in world
  @SideOnly(Side.CLIENT)
  @Override
  public int func_180662_a(IBlockAccess worldIn, BlockPos pos, int renderPass) {
    return 0xffffff;
  }

  @Override
  public boolean func_176200_f(World worldIn, BlockPos pos) {
    return false;
  }

  @Override
  protected boolean func_149854_a(Block ground) {
    return ground == TinkerWorld.slimeGrass || ground == TinkerWorld.slimeDirt;
  }

  @Override
  public EnumPlantType getPlantType(IBlockAccess world, BlockPos pos) {
    return TinkerWorld.slimePlantType;
  }

  @Override
  public int func_176222_j(World worldIn, BlockPos pos) {
    IBlockState iblockstate = worldIn.func_180495_p(pos);
    return iblockstate.func_177230_c().func_176201_c(iblockstate);
  }

  @Override
  public void func_176476_e(World worldIn, BlockPos pos, IBlockState state, Random rand) {
    if (!net.minecraftforge.event.terraingen.TerrainGen.saplingGrowTree(worldIn, rand, pos)) return;

    BlockSlime.SlimeType slimeType = BlockSlime.SlimeType.GREEN;
    if(state.func_177229_b(FOLIAGE) == FoliageType.ORANGE) {
      slimeType = BlockSlime.SlimeType.MAGMA;
    }

    IBlockState slimeGreen = TinkerWorld.slimeBlockCongealed.func_176223_P().func_177226_a(BlockSlime.TYPE, slimeType);
    IBlockState leaves = TinkerWorld.slimeLeaves.func_176223_P().func_177226_a(BlockSlimeGrass.FOLIAGE, state.func_177229_b(FOLIAGE));
    SlimeTreeGenerator gen = new SlimeTreeGenerator(5, 4, slimeGreen, leaves, null);

    // replace sapling with air
    worldIn.func_175698_g(pos);

    // try generating
    gen.generateTree(rand, worldIn, pos);

    // check if it generated
    if(worldIn.func_175623_d(pos)) {
      // nope, set sapling again
      worldIn.func_180501_a(pos, state, 4);
    }
  }
}
